@@ -8,6 +8,7 @@ from fastutils import rsautils
 from fastutils import strutils
 from fastutils import funcutils
 from fastutils import typingutils
+from fastutils import cipherutils
 
 from django.http import JsonResponse
 from django.conf import settings
@@ -15,10 +16,12 @@ from django.conf import settings
 from .utils import SimpleJsonEncoder
 from .utils import get_request_data
 from .pack import SimpleJsonResultPacker
+from .pack import SafeJsonResultPacker
 
 logger = logging.getLogger(__name__)
 
 simple_result_packer = SimpleJsonResultPacker()
+safe_result_packer = SafeJsonResultPacker()
 
 def setup_result_packer(packer):
     global simple_result_packer
@@ -45,18 +48,22 @@ class View(object):
                 return funcutils.call_with_inject(func, self.data)
 
 class Apiview(object):
-    def __init__(self, packer):
+    def __init__(self, packer, *preload_plugins):
         self.packer = packer
+        self.preload_plugins = preload_plugins
     
     def __call__(self, func):
         """Turn the view function into apiview function. Must use as the first decorator.
         """
         def wrapper(request, **kwargs):
+            target_func = func
             defaults = funcutils.get_default_values(func)
             view = View(request, defaults, **kwargs)
             package = {}
             try:
-                result = view.process(func)
+                for plugin in reversed(self.preload_plugins):
+                    target_func = plugin(target_func)
+                result = view.process(target_func)
                 package = self.packer.pack_result(result)
             except Exception as error:
                 logger.exception("apiview process failed: {}".format(str(error)))
@@ -69,6 +76,7 @@ class Apiview(object):
 
 apiview = Apiview(simple_result_packer)
 
+
 def requires(*parameter_names):
     """Throw bizerror.MissingParameter exception if required parameters not given.
     """
@@ -233,3 +241,40 @@ def cache(key, expire=None, cache_name="default", get_from_cache=None, set_to_ca
         return functools.wraps(func)(wrapper)
     return wrapper_outer
 
+def load_client_rsa_publickey(get_client_rsa_publickey, client_id_fieldname="clientId", client_rsa_publickey_fieldname="CLIENT_RSA_PUBLICKEY"):
+    def wrapper_outer(func):
+        def wrapper(_django_apiview_view_instance):
+            view = _django_apiview_view_instance
+            request = view.data["_request"]
+            client_id = view.data.get(client_id_fieldname, None)
+            if client_id:
+                publickey = get_client_rsa_publickey(client_id)
+                setattr(request, client_rsa_publickey_fieldname, publickey)
+            return view.process(func)
+        return functools.wraps(func)(wrapper)
+    return wrapper_outer
+
+def decode_encrypted_data(result_encoder=cipherutils.SafeBase64Encoder(), server_rsa_privatekey_filedname="RSA_PRIVATEKEY", encrypted_password_fieldname="encryptedPassword", encrypted_data_fieldname="encryptedData"):
+    def wrapper_outer(func):
+        def wrapper(_django_apiview_view_instance):
+            view = _django_apiview_view_instance
+            request = view.data["_request"]
+            privatekey = getattr(settings, server_rsa_privatekey_filedname, None)
+            encrypted_password = view.data.get(encrypted_password_fieldname, "")
+            encrypted_data = view.data.get(encrypted_data_fieldname, "")
+            if privatekey and encrypted_password and encrypted_data:
+                password = rsautils.decrypt(encrypted_password, privatekey)
+                cipher = cipherutils.AesCipher(password=password, result_encoder=result_encoder)
+                data_text = cipher.decrypt(encrypted_data)
+                data = json.loads(data_text)
+                view.data.update(data)
+            return view.process(func)
+        return functools.wraps(func)(wrapper)
+    return wrapper_outer
+
+def safe_apiview(packer_class=SafeJsonResultPacker, **kwargs):
+    return Apiview(
+        funcutils.call_with_inject(packer_class, kwargs),
+        funcutils.call_with_inject(load_client_rsa_publickey, kwargs),
+        funcutils.call_with_inject(decode_encrypted_data, kwargs),
+        )
